libname in 'R:\teaching\short course\missing data\real data\ATS';

proc contents data=in.nats_2013_finalcpl_6_weighted;
run;

OPTIONS nofmterr;

proc freq data=in.nats_2013_finalcpl_6_weighted;
tables statefips gender RACEETHNIC RACEETHNIC_R GENHEALTH MARITAL2 EDUCA2_R INCOME2 SMOK100 SMOKNOW EMPLOY2/missing list;
run;

/*Check data file*/
proc means data=in.nats_2013_finalcpl_6_weighted;
var age; 
run;

/*Recoding*/
data indat;
set in.nats_2013_finalcpl_6_weighted;
keep Age_R gender_R RACE_R GENHEALTH_R MARITAL2_R EDUCA2_R_R INCOME2_R SMOK100_R SMOKNOW_R EMPLOY2_R;
if gender<0 then gender_R=.;
else gender_R=gender;
if RACEETHNIC_R=6 then RACE_R=.;
else RACE_R=RACEETHNIC_R;
if genhealth<0 then genhealth_R=.;
else genhealth_R=genhealth;
if MARITAL2>6 OR MARITAL2<0 then MARITAL2_R=.;
else MARITAL2_R=MARITAL2;
if EDUCA2_R=7 then EDUCA2_R_R=.;
else EDUCA2_R_R=EDUCA2_R;
if INCOME2<0 then INCOME2_R=.;
else INCOME2_R=INCOME2;
if SMOK100<0 then SMOK100_R=.;
else SMOK100_R=SMOK100;
if SMOKNOW<0 then SMOKNOW_R=.;
else SMOKNOW_R=SMOKNOW;
if EMPLOY2<0 then EMPLOY2_R=.;
else EMPLOY2_R=EMPLOY2;
if Age<0 then Age_R=.;
else Age_R=Age;
run;

proc contents data=indat;
run;

proc freq data=indat;
tables gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R INCOME2_R SMOK100_R SMOKNOW_R EMPLOY2_R /missing list;
run;

proc univariate data=indat;
var age_r;
run;

/*Check missing data pattern and EM estimates*/
proc mi data=indat seed=1518971 simple nimpute=0;
      em itprint outem=outem;
      var age_R gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R INCOME2_R SMOK100_R SMOKNOW_R EMPLOY2_R;
   run;

/*Multiple imputation approach*/
proc mi data=indat nimpute=5 seed=789417 round=1 1 1 1 1 1 1 1 1 1 out=out_mi1;
class gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R INCOME2_R SMOK100_R SMOKNOW_R EMPLOY2_R;
fcs;
var age_R gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R EMPLOY2_R SMOK100_R INCOME2_R SMOKNOW_R; 
run;

data out_mi1_2;
set out_mi1;
if SMOK100_R=1 and (SMOKNOW_R=1 or SMOKNOW_R=2) then smk=1;
else smk=0;
run;

   proc logistic data=out_mi1_2 outest=ordinal_parms covout;
        by _imputation_;
        class gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R INCOME2_R EMPLOY2_R smk;
        model smk=age_r gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R INCOME2_R EMPLOY2_R;
        run;

		proc contents data=ordinal_parms;
		run;

   proc mianalyze data=ordinal_parms;
        modeleffects intercept age_r gender_R1 RACE_R1 RACE_R2 RACE_R3 RACE_R4
                     genhealth_R1 genhealth_R2 genhealth_R3 genhealth_R4
                     MARITAL2_R1 MARITAL2_R2 MARITAL2_R3 MARITAL2_R4 MARITAL2_R5 
                     EDUCA2_R_R1 EDUCA2_R_R2 EDUCA2_R_R3 EDUCA2_R_R4
                     INCOME2_R1 INCOME2_R2 INCOME2_R3 INCOME2_R4 INCOME2_R5 INCOME2_R6 INCOME2_R7 
                     EMPLOY2_R1;
   run;

/*Propensity score approach*/

/*Impute variables besides smoking status*/
proc mi data=indat nimpute=1 seed=209704 round=1 1 1 1 1 1 1 1 out=out_mi2;
class gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R EMPLOY2_R SMOK100_R SMOKNOW_R;
fcs;
var age_R gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R EMPLOY2_R SMOK100_R SMOKNOW_R; 
run;

data out_mi2_2;
set out_mi2;
if SMOK100_R=1 and (SMOKNOW_R=1 or SMOKNOW_R=2) then smk=1;
else smk=0;
if INCOME2_R=. then r=0;
else r=1;
where age_R^=.;
/*where age_R^=. and gender_R^=. RACE_R^=. genhealth_R^=. MARITAL2_R^=. EDUCA2_R_R^=. EMPLOY2_R^=. SMOK100_R^=. 
SMOKNOW_R^=.*/;
run;

proc freq data=out_mi2_2;
tables INCOME2_R*r r/missing list;
run;

/*Univariate model*/

%macro runlogit(var,index);
proc logistic data=out_mi2_2;
%if &index.=1 %then %do;
class r &var.;
%end;
%else %do;
class r;
%end;
model r=&var.;
run;
%mend runlogit;

%runlogit(age_R,2);
%runlogit(gender_R,1);
%runlogit(RACE_R,1);
%runlogit(genhealth_R,1);
%runlogit(MARITAL2_R,1);
%runlogit(EDUCA2_R_R,1);
%runlogit(EMPLOY2_R,1);
%runlogit(SMK,1);

/*Stepwise model selection*/
proc logistic data=out_mi2_2;
class gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R EMPLOY2_R;
model r(event='1')=age_r gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R EMPLOY2_R/selection=stepwise
                     slentry=0.3
                     slstay=0.35
                     details
                     lackfit;
	output out=pred p=phat lower=lcl upper=ucl
    predprob=(individual crossvalidate);
run;

proc contents data=pred;
run;

proc means P20 P40 P60 P80 data=pred;
var phat;
run;

data pred2;
set pred;
if phat<0.6770857 then WC1=1;
else if phat<0.7358703 then WC1=2;
else if phat<0.7726769 then WC1=3;
else if phat<0.8174769 then WC1=4;
else WC1=5;
run;

proc means data=pred2;
var r;
class WC1;
output out=out_wc mean=phat1;
run;

data out_wc;
set out_wc;
where _TYPE_^=0;
drop _TYPE_ _FREQ_;
run;

proc sort data=pred2;
by WC1;
run;

data pred3;
merge pred2(in=A) out_wc(in=B);
by WC1;
wt=1/phat;
wt1=1/phat1;
if A and B;
run;

proc freq data=pred3;
tables INCOME2_R /missing list;
where INCOME2_R^=.;
run;

proc surveyfreq data=pred3;
tables INCOME2_R;
weight wt;
run;

proc surveyfreq data=pred3;
tables INCOME2_R;
weight wt1;
run;

proc surveylogistic data=pred3;
class gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R INCOME2_R EMPLOY2_R smk;
        model smk=age_r gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R INCOME2_R EMPLOY2_R;
		weight wt;
run;

proc surveylogistic data=pred3;
class gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R INCOME2_R EMPLOY2_R smk;
        model smk=age_r gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R INCOME2_R EMPLOY2_R;
		weight wt1;
run;

data out_wc;
set pred3;
keep smk r age_r gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R EMPLOY2_R INCOME2_R;
run;

proc contents data=out_wc;
run;

proc freq data=out_wc;
tables age_r gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R EMPLOY2_R INCOME2_R /missing list;
run;

proc export data=out_wc outfile='R:\teaching\short course\missing data\real data\ATS\ats.csv' dbms = csv replace;
run;

proc import datafile="R:\teaching\short course\missing data\session3\simulation\outdat.csv" 
out=mydata dbms=csv replace; 
getnames=yes; 
run;

data mydata2;
set mydata;
wt2=1/ep2;
run;

proc surveyfreq data=mydata2;
tables INCOME2_R;
weight wt2;
run;

proc surveylogistic data=mydata2;
class gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R INCOME2_R EMPLOY2_R smk;
        model smk=age_r gender_R RACE_R genhealth_R MARITAL2_R EDUCA2_R_R INCOME2_R EMPLOY2_R;
		weight wt2;
run;
